@@ -5,18 +5,18 @@ from nameko.containers import WorkerContext, PARENT_CALLS_KEY
 from nameko.events import event_handler, event_dispatcher, Event as NamekoEvent
 from nameko.rpc import rpc, rpc_proxy
 from nameko.testing.utils import (
-    get_container, wait_for_call, worker_context_factory)
+    get_container, wait_for_call, worker_context_factory, DummyProvider)
 from nameko.testing.services import entrypoint_hook
 
 
 def get_logging_worker_context(stack_request):
     class LoggingWorkerContext(WorkerContext):
-        def __init__(self, container, service, method_name, args=None,
+        def __init__(self, container, service, provider, args=None,
                      kwargs=None, data=None):
             parent_stack = data.get('call_id_stack') if data else None
             stack_request(parent_stack)
             super(LoggingWorkerContext, self).__init__(
-                container, service, method_name, args, kwargs, data
+                container, service, provider, args, kwargs, data
             )
     return LoggingWorkerContext
 
@@ -31,20 +31,20 @@ def test_worker_context_gets_stack(container_factory):
     container = container_factory(FooService, {})
     service = FooService()
 
-    context = context_cls(container, service, "bar")
+    context = context_cls(container, service, DummyProvider("bar"))
     assert context.call_id == 'baz.bar.0'
     assert context.call_id_stack == ['baz.bar.0']
     assert context.parent_call_stack == []
 
     # Build stack
-    context = context_cls(container, service, "foo",
+    context = context_cls(container, service, DummyProvider("foo"),
                           data={'call_id_stack': context.call_id_stack})
     assert context.call_id == 'baz.foo.1'
     assert context.call_id_stack == ['baz.bar.0', 'baz.foo.1']
 
     # Long stack
     many_ids = [str(i) for i in xrange(10)]
-    context = context_cls(container, service, "long",
+    context = context_cls(container, service, DummyProvider("long"),
                           data={'call_id_stack': many_ids})
     expected = many_ids + ['baz.long.2']
     assert context.call_id_stack == expected
@@ -62,7 +62,7 @@ def test_short_call_stack(container_factory):
 
     # Trim stack
     many_ids = [str(i) for i in xrange(100)]
-    context = context_cls(container, service, "long",
+    context = context_cls(container, service, DummyProvider("long"),
                           data={'call_id_stack': many_ids})
     assert context.call_id_stack == ['99', 'baz.long.0']
 
